#ifndef __OBJECT_FUNCTION_H__
#define __OBJECT_FUNCTION_H__

#include "Object.h"
#include "Expr.h"

namespace AScript {

//-----------------------------------------------------------------------------
// Class_Function / Object_Function
//-----------------------------------------------------------------------------
class DLLEXPORT Class_Function : public Class {
public:
	Class_Function(Environment &env);
	void Prepare();
	virtual Object *CreateDescendant(Environment &env, Signal sig, Class *pClass);
};

class DLLEXPORT Object_Function : public Object {
public:
	inline static Object_Function *GetSelfObj(Context &context) {
		return dynamic_cast<Object_Function *>(context.GetSelfObj());
	}
protected:
	Function *_pFunc;
public:
	inline Object_Function(Class *pClass, Function *pFunc) :
									Object(pClass), _pFunc(pFunc) {}
	inline Object_Function(const Object_Function &obj) :
									Object(obj), _pFunc(obj._pFunc->IncRef()) {}
	~Object_Function();
	virtual bool IsFunction() const;
	virtual Object *Clone() const;
	inline Function *GetFunction() { return _pFunc; }
	inline const Function *GetFunction() const { return _pFunc; }
	inline String GetName() const { return _pFunc->GetName(); }
	virtual String GetFullName(Signal sig);
	virtual String ToString(Signal sig, bool exprFlag);
	virtual Value Call(Environment &env, Signal sig, ContextExpr &contextExpr);
	virtual Value Eval(Environment &env, Signal sig, ValueList &valListArg) const;
};

class DLLEXPORT Object_WrappedMethod : public Object_Function {
public:
	inline static Object_WrappedMethod *GetSelfObj(Context &context) {
		return dynamic_cast<Object_WrappedMethod *>(context.GetSelfObj());
	}
protected:
	Value _valueSelf;
public:
	inline Object_WrappedMethod(Class *pClass, Function *pFunc, const Value &valueSelf) :
							Object_Function(pClass, pFunc), _valueSelf(valueSelf) {}
	inline Object_WrappedMethod(const Object_WrappedMethod &obj) :
							Object_Function(obj), _valueSelf(obj._valueSelf) {}
	~Object_WrappedMethod();
	virtual bool IsWrappedMethod() const;
	virtual Object *Clone() const;
	virtual String GetFullName(Signal sig);
	virtual String ToString(Signal sig, bool exprFlag);
	virtual Value Call(Environment &env, Signal sig, ContextExpr &contextExpr);
	virtual Value Eval(Environment &env, Signal sig, ValueList &valListArg) const;
	String MakePrefix(Signal sig) const;
};

}

#endif
